/*----------------------------------------------------------------------------+
| Library Name: SETDTTM.PLB                                                   |
|                                                                             |
| Description:  Contains two functions SETFDATE & SETFTIME                    |
|                                                                             |
|                                                                             |
|     Function: SETFDATE                                                      |
|    -------------------                                                      |
|                                                                             |
|      Purpose: Sets a file(s) date(s)                                        |
|                                                                             |
|       Syntax: SETFDATE(<expC>,<expN1>,<expN2>,<expN3>)                      |
|                                                                             |
|   Input                                                                     |
|   Parameters: <expC>  - the name of the file whose date we want to set.     |
|               <expN1> - the year (It must be entered with the century!)     |
|               <expN2> - the month                                           |
|               <expN3> - the day                                             |
|                                                                             |
|   Output                                                                    |
|   Parameters: <expL1> - returns a .T. if the function call was successful;  |
|                         otherwise, a .F. is returned.                       |
|                                                                             |
|    Example 1: SETFDATE("TEST.PRG", 1993, 12, 30)                            |       |
|               - sets the date on the file TEST.PRG in the current           |
|                 directory to: Dec. 30, 1993 or 12-30-93                     |
|                                                                             |
|                                                                             |
|   Misc Notes: The file name can contain a drive and path. <expC> can be     |
|               either upper or lower case.                                   |
|                                                                             |
|                                                                             |
|                                                                             |
|                                                                             |
|     Function: SETFTIME                                                      |
|    -------------------                                                      |
|                                                                             |
|      Purpose: Sets a file(s) time(s)                                        |
|                                                                             |
|       Syntax: SETFTIME(<expC>,<expN1>,<expN2>,<expN3>)                      |
|                                                                             |
|   Input                                                                     |
|   Parameters: <expC>  - the name of the file whose time we want to set.     |
|               <expN1> - the hour (It must be entered in military time!)     |
|               <expN2> - the minute                                          |
|               <expN3> - the second                                          |
|                                                                             |
|   Output                                                                    |
|   Parameters: <expL1> - returns a .T. if the function call was successful;  |
|                         otherwise, a .F. is returned.                       |
|                                                                             |
|    Example 1: SETFTIME("TEST.PRG", 17, 15, 30)                              |       |
|               - sets the time on the file TEST.PRG in the current           |
|                 directory to: 05:15:30 pm                                   |
|                                                                             |
|                                                                             |
|   Misc Notes: The file name can contain a drive and path. <expC> can be     |
|               either upper or lower case.                                   |
+-----------------------------------------------------------------------------*/
#include <string.h>
#include <stdio.h>
#include <dos.h>
#include <fcntl.h>
#include "pro_ext.h"

#define YEAR(t)    (((t & 0xFE00) >> 9) + 1980)
#define MONTH(t)   ((t & 0x01E0) >> 5)
#define DAY(t)     (t & 0x001F)
#define HOUR(t)    ((t & 0xF800) >> 11)
#define MINUTE(t)  ((t & 0x07E0) >> 5)
#define SECOND(t)  ((t & 0x001F) << 1)

void FAR setfdate(ParamBlk FAR *parm)
{
    int handle,
        retcode;

    unsigned short month,
                   day,
                   year,
                   date,
                   time;


    char FAR * filename;

    _SetHandSize(parm->p[0].val.ev_handle, parm->p[0].val.ev_length+1);
    _HLock(parm->p[0].val.ev_handle);
    filename = _HandToPtr(parm->p[0].val.ev_handle);
    filename[parm->p[0].val.ev_length] = '\0';

    year  = parm->p[1].val.ev_long;
    month = parm->p[2].val.ev_long;
    day   = parm->p[3].val.ev_long;

    /* Assumes the year is coming in with the century */
    year = year - 1980;

    if ( _dos_open( filename, O_RDONLY, &handle) != 0 )
        retcode = 0;
    else {
       _dos_getftime( handle, &date, &time);
       date = (year << 9) + (month << 5) + day;
       _dos_setftime( handle, date, time);
       retcode = 1;
       _dos_close(handle);
    }

    _HUnLock(parm->p[0].val.ev_handle);
    _RetLogical(retcode);
    return;
}


void FAR setftime(ParamBlk FAR *parm)
{
    int handle,
        retcode;

    unsigned short hour,
                   min,
                   sec,
                   date,
                   time;

    char FAR * filename;

    _SetHandSize(parm->p[0].val.ev_handle, parm->p[0].val.ev_length+1);
    _HLock(parm->p[0].val.ev_handle);
    filename = _HandToPtr(parm->p[0].val.ev_handle);
    filename[parm->p[0].val.ev_length] = '\0';

    hour = parm->p[1].val.ev_long;
    min  = parm->p[2].val.ev_long;
    sec  = parm->p[3].val.ev_long;

    if ( _dos_open( filename, O_RDONLY, &handle) != 0 )
        retcode = 0;
    else {
        _dos_getftime( handle, &date, &time);
        time = (hour << 11) + (min << 5) + (sec >> 1);
        _dos_setftime( handle, date, time);
        _dos_close(handle);
        retcode = 1;
    }

    _HUnLock(parm->p[0].val.ev_handle);
    _RetLogical(retcode);
    return;
}


FoxInfo myFoxInfo[] =
{
    {"SETFDATE", setfdate, 4, "C,I,I,I"},
    {"SETFTIME", setftime, 4, "C,I,I,I"}
};


FoxTable _FoxTable =
{
    (FoxTable FAR *)0, sizeof(myFoxInfo) / sizeof(FoxInfo), myFoxInfo
};
